﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SearchEngineListEditorLibrary;
using Hidori.Windows.Forms;

namespace SearchEngineListEditorApp
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		bool initialized = false;

		private void treeViewFolders_AfterSelect( object sender, TreeViewEventArgs e )
		{
			if ( initialized )
			{
				this.core.FolderTreeAfterSelect();
			}
		}

		private EditorCore core;

		private void Initialize()
		{
			core = new EditorCore( 
				this.treeViewFolders, 
				this.listViewCurrentItems, 
				(ToolStripStatusLabel)this.statusStrip1.Items[0] );

			initialized = true;

			UpdateView();
		}

		private void UpdateView()
		{
			this.core.Update( ( Item )this.treeViewFolders.TopNode.Tag );
		}

		#region ToolStripButton Click Methods
		private void toolStripButtonNewSearchEngine_Click( object sender, EventArgs e )
		{
			this.core.NewSearchEngine();
		}

		private void toolStripButtonNewFolder_Click( object sender, EventArgs e )
		{
			this.core.NewFolder();
		}

		private void toolStripButtonNewGroup_Click( object sender, EventArgs e )
		{
			this.core.NewGroup();
		}

		private void toolStripButtonNewBar_Click( object sender, EventArgs e )
		{
			this.core.NewBar();
		}

		private void toolStripButtonCopy_Click( object sender, EventArgs e )
		{
			this.core.CopyItems();
		}

		private void toolStripButtonRemoveItems_Click( object sender, EventArgs e )
		{
			this.core.RemoveSelectedItems();
		}

		private void toolStripButtonMoveUp_Click( object sender, EventArgs e )
		{
			this.core.MoveUpSelectedItems();
		}

		private void toolStripButtonMoveDown_Click( object sender, EventArgs e )
		{
			this.core.MoveDownSelectedItems();
		}
		#endregion

		#region ToolStripMenuItem Click Methods
		private void toolStripMenuItemOpen_Click( object sender, EventArgs e )
		{
			this.core.OpenFile();
		}

		private void toolStripMenuItemSave_Click( object sender, EventArgs e )
		{
			this.core.SaveFile();
		}

		private void toolStripMenuItemOverWriteSave_Click( object sender, EventArgs e )
		{
			this.core.OverWriteSaveFile();
		}

		private void toolStripMenuItemExit_Click( object sender, EventArgs e )
		{
			this.core.Exit();
		}

		private void toolStripMenuItemNewSearchEngine_Click( object sender, EventArgs e )
		{
			this.core.NewSearchEngine();
		}

		private void toolStripMenuItemNewFolder_Click( object sender, EventArgs e )
		{
			this.core.NewFolder();
		}

		private void toolStripMenuItemNewGroup_Click( object sender, EventArgs e )
		{
			this.core.NewGroup();
		}

		private void toolStripMenuItemNewBar_Click( object sender, EventArgs e )
		{
			this.core.NewBar();
		}

		private void toolStripMenuItemCopy_Click( object sender, EventArgs e )
		{
			this.core.CopyItems();
		}

		private void toolStripMenuItemRemove_Click( object sender, EventArgs e )
		{
			this.core.RemoveSelectedItems();
		}

		private void toolStripMenuItemMoveUp_Click( object sender, EventArgs e )
		{
			this.core.MoveUpSelectedItems();
		}

		private void toolStripMenuItemMoveDown_Click( object sender, EventArgs e )
		{
			this.core.MoveDownSelectedItems();
		}

		private void toolStripMenuItemVersion_Click( object sender, EventArgs e )
		{
			ShowVersion();
		}

		#endregion

		#region ShowVersion
		private const string version = "1.11";

		private static void ShowVersion()
		{
			MessageBox.Show( MainForm.version );
		} 
		#endregion

		private void MainForm_Load( object sender, EventArgs e )
		{
			Initialize();
		}

		private void listViewCurrentItems_MouseDoubleClick( object sender, MouseEventArgs e )
		{
			ListViewItem listViewItem = this.listViewCurrentItems.GetItemAt( e.X, e.Y );

			if ( listViewItem == null )
			{
				return;
			}

			Item item = ( Item )listViewItem.Tag;

			if ( item is Folder )
			{
				SelectNodeWithFolder( ( Folder )item );
			}
			else
			{
				this.core.EditItem();
			}
		}

		private Folder findTarget;

		private void SelectNodeWithFolder( Folder folder )
		{
			findTarget = folder;

			TreeNode node = this.treeViewFolders.Nodes.FindRecursive( HasTargetFolder );

			this.treeViewFolders.SelectedNode = node;
		}

		private bool HasTargetFolder( TreeNode node )
		{
			return node.Tag == findTarget;
		}

		private void toolStripMenuItemShotCutList_Click( object sender, EventArgs e )
		{
			this.core.ShowShortCutList();
		}

		private void contextMenuStripCurrentItems_Click( object sender, EventArgs e )
		{
			this.core.EditItem();
		}

		private void listViewCurrentItems_ItemDrag( object sender, ItemDragEventArgs e )
		{
			if ( e.Button == MouseButtons.Left )
			{
				List<Item> draggedItems = this.core.GetSelectedItems();

				DoDragDrop( draggedItems, DragDropEffects.Move );
			}

		}

		private void DragEnterToTreeOrList( object sender, DragEventArgs e )
		{
			if ( !e.Data.GetDataPresent( typeof( List<Item> ) ) )
			{
				e.Effect = DragDropEffects.None;
			}
			else
			{
				e.Effect = e.AllowedEffect;
			}
		}

		private void treeViewFolders_DragDrop( object sender, DragEventArgs e )
		{
			TreeNode targetNode = GetMouseOverNode( e );

			if ( targetNode == null )
			{
				return;
			}

			Folder destination = ( Folder )targetNode.Tag;

			DragDropToTreeOrList( e, destination );
		}

		private static void DragDropToTreeOrList( DragEventArgs e, Folder destination )
		{
			List<Item> draggedItems = ( List<Item> )e.Data.GetData( typeof( List<Item> ) );

			try
			{
				foreach ( Item item in draggedItems )
				{
					ItemTree.MoveItem( item, destination );
				}
			}
			catch ( ArgumentNullException error )
			{
				ShowDropErrorMessage( "ArgumentNullException  " + error.Message );
			}
			catch ( ArgumentException error )
			{
				ShowDropErrorMessage( error.Message );
			}
		}

		private static void ShowDropErrorMessage( string message )
		{
			MessageBox.Show( "ドロップできません。" + "\n" + message );
		}

		private void treeViewFolders_DragOver( object sender, DragEventArgs e )
		{
			TreeNode mouseOverNode = GetMouseOverNode( e );

			if ( mouseOverNode != null )
			{
				treeViewFolders.SelectedNode = mouseOverNode;
			}
		}

		private TreeNode GetMouseOverNode( DragEventArgs e )
		{
			Point targetPoint = treeViewFolders.PointToClient( new Point( e.X, e.Y ) );

			return treeViewFolders.GetNodeAt( targetPoint );
		}

		private void treeViewFolders_ItemDrag( object sender, ItemDragEventArgs e )
		{
			TreeNode draggedNode = ( TreeNode )e.Item;

			if ( draggedNode == treeViewFolders.TopNode )
			{
				return;
			}

			if ( e.Button == MouseButtons.Left )
			{
				List<Item> draggedItems = new List<Item>();

				draggedItems.Add( ( Item )draggedNode.Tag );

				DoDragDrop( draggedItems, DragDropEffects.Move );
			}
		}

		private void listViewCurrentItems_DragDrop( object sender, DragEventArgs e )
		{
			Point clientPoint = this.listViewCurrentItems.PointToClient( new Point( e.X, e.Y ) );

			ListViewItem item =
				this.listViewCurrentItems.GetItemAt( clientPoint.X, clientPoint.Y );

			Folder destination;

			if ( item != null &&
				item.Tag is Folder )
			{
				destination = ( Folder )item.Tag;
			}
			else
			{
				destination = ( Folder )this.treeViewFolders.SelectedNode.Tag;
			}

			DragDropToTreeOrList( e, destination );
		}
	}
}
