﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SearchEngineListEditorLibrary;

namespace SearchEngineListEditorLibrary
{
	public partial class EditForm : Form
	{
		public EditForm( Item item )
		{
			InitializeComponent();

			this.item = item;

			InitializeItemDetailView();
		}

		private void InitializeItemDetailView()
		{
			this.textBoxName.Text = item.Name;
			this.textBoxURLFirst.Text = item.URLFirst;
			this.textBoxURLLast.Text = item.URLLast;
			this.comboBoxEncode.SelectedIndex = item.Encode;
			this.comboBoxShortCut.SelectedIndex = item.ShortCut;
			this.checkBoxLock.Checked = item.Lock;

			if ( item is Group )
			{
				Group group = ( Group )item;

				this.textBoxGroupMember.Lines = group.SearchEngineNames.ToArray();
			}
		}

		private Item item;

		private void buttonOK_Click( object sender, EventArgs e )
		{
			AdjustValues();

			if ( AreValidValues() )
			{
				UpdateItem();

				this.DialogResult = DialogResult.OK;
			}
			else
			{
				MessageBox.Show( "名前を入力して下さい。" );
			}
		}

		private bool AreValidValues()
		{
			return this.textBoxName.Text != string.Empty;
		}

		private void AdjustValues()
		{
			TrimTexts();

			AdjustGroopMembers();
		}

		private void AdjustGroopMembers()
		{
			TrimAndRemoveEmptyLines( this.textBoxGroupMember );
		}

		private void TrimAndRemoveEmptyLines( TextBox textBox )
		{
			List<string> newLines = new List<string>();

			foreach ( string text in textBox.Lines )
			{
				string trimmedText = text.Trim();

				if ( trimmedText != string.Empty )
				{
					newLines.Add( trimmedText );
				}
			}

			textBox.Lines = newLines.ToArray();
		}

		private void TrimTexts()
		{
			TrimText( this.textBoxName );
			TrimText( this.textBoxURLFirst );
			TrimText( this.textBoxURLLast );
		}

		private void TrimText( TextBox textBox )
		{
			textBox.Text = textBox.Text.Trim();
		}

		private void UpdateItem()
		{
			item.SetProperties(
				this.textBoxName.Text,
				this.textBoxURLFirst.Text,
				this.textBoxURLLast.Text,
				this.checkBoxLock.Checked,
				this.comboBoxEncode.SelectedIndex,
				this.comboBoxShortCut.SelectedIndex,
				item.Icon );

			if ( item is Group )
			{
				Group group = ( Group )item;

				group.SearchEngineNames = this.textBoxGroupMember.Lines.ToList<string>();
			}
		}

		internal void SetSearchEngineEditMode()
		{
			this.tableLayoutPanelGroupMember.Enabled = false;
		}

		internal void SetFolderEditMode()
		{
			this.tableLayoutPanelURLFirst.Enabled = false;
			this.tableLayoutPanelURLLast.Enabled = false;
			this.tableLayoutPanelEncode.Enabled = false;
			this.tableLayoutPanelShortCut.Enabled = false;
			this.checkBoxLock.Enabled = false;
			this.tableLayoutPanelGroupMember.Enabled = false;
		}

		internal void SetGroupEditMode()
		{
			this.tableLayoutPanelURLFirst.Enabled = false;
			this.tableLayoutPanelURLLast.Enabled = false;
			this.tableLayoutPanelEncode.Enabled = false;

			this.checkBoxLock.Enabled = false;
		}
	}
}
