﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Tomham.Collections.Specialized;
using Tomham.IO;

namespace SearchEngineListEditorLibrary
{
	public class IniFileWriter
	{
		public void SaveToFile( string fileName, Folder root )
		{
			SearchEngineIniFile iniFile = new SearchEngineIniFile();

			List<Item> rawItems = GetRawItems( root );

			InitializeFile( fileName );

			WriteRawItems( rawItems, iniFile );

			iniFile.SaveToFile( fileName );
		}

		private void WriteRawItems( List<Item> rawItems, SearchEngineIniFile iniFile )
		{
			iniFile.WriteItemCount( rawItems.Count );

			for ( int itemNumber = 0; itemNumber < rawItems.Count; itemNumber++ )
			{
				Item rawItem = rawItems[itemNumber];

				WriteRawItem( iniFile, rawItem, itemNumber );
			}
		}

		private void WriteRawItem( SearchEngineIniFile iniFile, Item rawItem, int itemNumber )
		{
			iniFile.WriteName( itemNumber, CreateRawItemString( rawItem.Name ) );
			iniFile.WriteURLFirst( itemNumber, CreateRawItemString( rawItem.URLFirst ) );
			iniFile.WriteURLLast( itemNumber, CreateRawItemString( rawItem.URLLast ) );
			iniFile.WriteLock( itemNumber, rawItem.Lock );
			iniFile.WriteEncode( itemNumber, rawItem.Encode );
			iniFile.WriteShotcut( itemNumber, rawItem.ShortCut );
			iniFile.WriteIcon( itemNumber, rawItem.Icon );
		}

		private string CreateRawItemString( string value )
		{
			if ( value == string.Empty )
			{
				return value;
			}
			else
			{
				return PathUtils.DoubleQuotString( value );
			}
		}

		private List<Item> GetRawItems( Folder root )
		{
			List<Item> rawItems = new List<Item>();

			foreach ( Item item in root )
			{
				item.CreateAndAddRawItems( rawItems );
			}

			return rawItems;
		}

		private void InitializeFile( string fileName )
		{
			if ( File.Exists( fileName ) )
			{
				ClearFile( fileName );
			}
			else
			{
				CreateFile( fileName );
			}
		}

		private void CreateFile( string fileName )
		{
			StringList file = new StringList();

			file.SaveToFile( fileName, Encoding.Default );
		}

		private void ClearFile( string fileName )
		{
			StringList file = new StringList( fileName, Encoding.Default );

			file.Clear();

			file.SaveToFile( fileName, Encoding.Default );
		}
	}
}
