﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SearchEngineListEditorLibrary
{
	public class Item: ICloneable
	{
		public Item()
		{
		}

		private string name = "NoName";

		public string Name
		{
			get { return name; }
			set
			{
				name = value;
				NotifyToOwner();
			}
		}

		private string urlFirst = string.Empty;

		public string URLFirst
		{
			get { return urlFirst; }
			set
			{
				urlFirst = value;
				NotifyToOwner();
			}
		}

		private string urlLast = string.Empty;

		public string URLLast
		{
			get { return urlLast; }
			set
			{
				urlLast = value;
				NotifyToOwner();
			}
		}

		private bool _lock;

		public bool Lock
		{
			get { return _lock; }
			set
			{
				_lock = value;
				NotifyToOwner();
			}
		}

		private int encode;

		public int Encode
		{
			get { return encode; }
			set
			{
				encode = value;
				NotifyToOwner();
			}
		}

		private int shortCut;

		public int ShortCut
		{
			get { return shortCut; }
			set
			{
				shortCut = value;
				NotifyToOwner();
			}
		}

		private int icon;

		public int Icon
		{
			get { return icon; }
			set
			{
				icon = value;
				NotifyToOwner();
			}
		}

		private ItemTree owner;

		public ItemTree Owner
		{
			get { return owner; }
		}

		internal virtual void SetOwner( ItemTree owner )
		{
			this.owner = owner;
		}

		private Folder parent;

		public Folder Parent
		{
			get { return parent; }
		}

		internal void SetParent( Folder parent )
		{
			this.parent = parent;
		}

		public void SetProperties( string name, string urlFirst, string urlLast,
			bool _lock, int encode, int shortCut, int icon )
		{
			this.DoSetProperties( name, urlFirst, urlLast, _lock, encode, shortCut, icon );

			NotifyToOwner();
		}

		private void DoSetProperties( string name, string urlFirst, string urlLast,
			bool _lock, int encode, int shortCut, int icon )
		{
			this.name = name;
			this.urlFirst = urlFirst;
			this.urlLast = urlLast;
			this._lock = _lock;
			this.encode = encode;
			this.shortCut = shortCut;
			this.icon = icon;
		}

		protected void NotifyToOwner()
		{
			if ( this.owner != null )
			{
				this.owner.Update( this );
			}
		}

		public virtual void CopyTo( Item destination )
		{
			destination.SetProperties(
				this.Name,
				this.URLFirst,
				this.URLLast,
				this.Lock,
				this.Encode,
				this.ShortCut,
				this.Icon );
		}

		public virtual void CreateAndAddRawItems( List<Item> rawItems )
		{
			Item rawItem = new Item();

			this.CopyTo( rawItem );

			rawItems.Add( rawItem );
		}

		public override string ToString()
		{
			return this.Name;
		}

		public bool IsAncester( Folder folder )
		{
			if ( this.Parent == null )
			{
				return false;
			}
			else if ( this.Parent == folder )
			{
				return true;
			}
			else
			{
				return this.Parent.IsAncester( folder );
			}
		}

		#region ICloneable メンバ

		public virtual object Clone()
		{
			Item result = new Item();

			result.SetProperties(
				this.Name,
				this.URLFirst,
				this.URLLast,
				this.Lock,
				this.Encode,
				this.ShortCut,
				this.Icon 
				);

			return result;
		}

		#endregion
	}
}
