﻿using System;
using NUnit.Framework;
using NUnit.Extensions.Converter;
using SearchEngineListEditorLibrary;
using System.IO;

namespace SearchEngineListEditorLibraryTest
{
	[TestFixture]
	public class ItemTreeTest
	{
		static string testFilesFolder = @"..\..\TestFiles";
		static string testFileName = Path.Combine( testFilesFolder, "test.ini" );

		[Test]
		public void SaveAndLoadWithFile()
		{
			Converter converter = new DefaultConverter();

			converter = new BeanValueConverter( converter, typeof( Item ), "Name", "URLFirst",
				"URLLast", "Lock", "Encode", "ShortCut", "Icon" );

			ItemTree itemTree = CreateItemTree();

			itemTree.SaveToFile( testFileName );

			ItemTree otherTree = new ItemTree();

			otherTree.LoadFromFile( testFileName );

			Assert.AreEqual( 2, otherTree.Root.Count );

			object expected =
				E.a( "item1Name", "item1URLFirst", "item1URLLast", true, 2, 3, 4 );

			converter.AssertEquals( expected, otherTree.Root[0] );

			Assert.IsTrue( otherTree.Root[0] is SearchEngine );

			Folder folder1 = ( Folder )otherTree.Root[1];

			Assert.AreEqual( "folder1", folder1.Name );

			Assert.AreEqual( 3, folder1.Count );

			Assert.IsTrue( folder1[0] is Bar );

			Assert.IsTrue( folder1[1] is Group );
		}

		private ItemTree CreateItemTree()
		{
			ItemTree tree = new ItemTree();

			SearchEngine item1 = new SearchEngine();

			item1.SetProperties(
				"item1Name", "item1URLFirst", "item1URLLast", true, 2, 3, 4 );

			Folder folder1 = new Folder { Name = "folder1" };

			Bar childBar = new Bar();

			Group childGroup = new Group();

			childGroup.SetProperties(
				"childGroupName", string.Empty, string.Empty, false, 0, 3, 0 );

			SearchEngine childItem = new SearchEngine();
			childItem.SetProperties(
				"childItemName", "childItemURLFirst", "childItemURLLast", false, 3, 4, 0 );

			folder1.Add( childBar );
			folder1.Add( childGroup );
			folder1.Add( childItem );

			tree.Root.Add( item1 );
			tree.Root.Add( folder1 );

			return tree;
		}

		[Test]
		public void MoveItem()
		{
			ItemTree tree = new ItemTree();

			Item item = new Item();

			Folder folder = new Folder();

			tree.Root.Add( folder );
			tree.Root.Add( item );

			Assert.AreEqual( tree.Root, item.Parent );
			Assert.AreEqual( 2, tree.Root.Count );
			Assert.AreEqual( 0, folder.Count );

			ItemTree.MoveItem( item, folder );

			Assert.AreEqual( folder, item.Parent );

			Assert.AreEqual( 1, tree.Root.Count );
			Assert.AreEqual( 1, folder.Count );
		}

		[Test, ExpectedException( typeof( ArgumentException ) )]
		public void MoveItemToChild()
		{
			Folder parent = new Folder();

			Folder child = new Folder();

			parent.Add( child );

			ItemTree.MoveItem( parent, child );
		}

		[Test, ExpectedException( typeof( ArgumentException ) )]
		public void MoveItemToGrandChild()
		{
			Folder parent = new Folder();

			Folder child = new Folder();

			Folder grandChild = new Folder();

			parent.Add( child );

			child.Add( grandChild );

			ItemTree.MoveItem( parent, grandChild );
		}

		[Test, ExpectedException( typeof( ArgumentException ) )]
		public void MoveItemToParent()
		{
			Folder parent = new Folder();

			Folder child = new Folder();

			parent.Add( child );

			ItemTree.MoveItem( child, parent );
		}

		[Test, ExpectedException( typeof( ArgumentException ) )]
		public void MoveItemToSelf()
		{
			Folder folder = new Folder();

			ItemTree.MoveItem( folder, folder );
		}
	}
}
