using System;
using System.IO;
using System.Text.RegularExpressions;
using System.Text;

namespace Tomham.IO
{
	/// <summary>
	/// FileSystem⏕邽߂Static Method񋟂
	/// </summary>
	public static class FileUtils
	{
		public static bool FileOrDirectoryExists(string path)
		{
			return ( File.Exists(path) || Directory.Exists(path) );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="path1"></param>
		/// <param name="path2"></param>
		/// <returns>0菬 path1LastWriteTime &lt; path2LastWriteTime</returns>
		public static int CompareLastWriteTime(string path1, string path2)
		{
			if ( path1 == null )
			{
				throw new ArgumentNullException("path1");
			}

			if ( path2 == null )
			{
				throw new ArgumentNullException("path2");
			}

			if ( ! FileOrDirectoryExists(path1) )
			{
				throw new ArgumentException("path1݂܂");
			}

			if ( ! FileOrDirectoryExists(path2) )
			{
				throw new ArgumentException("path2݂܂");
			}

			return DateTime.Compare( 
				File.GetLastWriteTime(path1),
				File.GetLastWriteTime(path2)
			);
		}

		/// <summary>
		/// Directoryۂ肷 DriveDirectoryƂ
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		public static bool IsDirectory(string path)
		{
			return (
				( File.GetAttributes(path) & FileAttributes.Directory) == 
				FileAttributes.Directory
			);
		}

		public static bool IsDrive(string path)
		{
			Regex regex = new Regex(@"^[a-z]:(\\)?$", RegexOptions.IgnoreCase);

			return regex.IsMatch(path);
		}

		/// <summary>
		/// Rs[悪ǂݎpłRs[
		/// </summary>
		/// <param name="sourcePath"></param>
		/// <param name="destPath"></param>
		public static void CopyFile(string sourcePath, string destPath)
		{
			if ( File.Exists(destPath) )
			{
				if ( HasSystemAttribute(destPath) || 
					HasReadOnlyAttribute(destPath) ||
					HasHiddenAttribute(destPath)
					)
				{
					FileUtils.ChangeAttributesToWritable( destPath );	
				}
			}

			File.Copy( sourcePath, destPath, true );
			CopyAttributes( sourcePath, destPath );
		}

		public static void CopyAttributes(string sourcePath, string destPath)
		{
			FileAttributes sourceAttributes = File.GetAttributes(sourcePath);
			File.SetAttributes( destPath, sourceAttributes );
		}

		public static bool HasReadOnlyAttribute(string path)
		{
			return ( File.GetAttributes(path) & FileAttributes.ReadOnly ) == 
				FileAttributes.ReadOnly;
		}

		public static bool HasHiddenAttribute(string path){
			return ( File.GetAttributes(path) & FileAttributes.Hidden ) == 
				FileAttributes.Hidden;
		}

		public static bool HasSystemAttribute(string path)
		{
			return ( File.GetAttributes(path) & FileAttributes.System ) == 
				FileAttributes.System;
		}
		
		/// <summary>
		/// ύXď㏑\ɂ
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		public static void ChangeAttributesToWritable(string path)
		{
			int tmp = Convert.ToInt32( File.GetAttributes(path) );

			if ( HasReadOnlyAttribute(path) )
			{
				tmp -= Convert.ToInt32( FileAttributes.ReadOnly ); 
			}

			if ( HasHiddenAttribute(path) )
			{
				tmp -= Convert.ToInt32( FileAttributes.Hidden );
			}

			if ( HasSystemAttribute(path) )
			{
				tmp -= Convert.ToInt32( FileAttributes.System );
			}

			FileAttributes newAttributes = (FileAttributes)(tmp);

			File.SetAttributes( path, newAttributes );
		}

		/// <summary>
		/// ǂݎptĂt@Cł폜
		/// </summary>
		/// <param name="path"></param>
		public static void DeleteFile(string path)
		{
			if ( path == null)
			{
				throw new ArgumentNullException("path");
			}

			if ( FileUtils.IsDirectory(path) )
			{
				throw new ArgumentException("pathDirectorył");
			}

			if ( !File.Exists(path) )
			{
				return;
			}

			FileUtils.ChangeAttributesToWritable( path );
			File.Delete(path);
		}

		/// <summary>
		/// ǂݎpƂAg̓Directorył폜
		/// </summary>
		/// <param name="path"></param>
		public static void DeleteDirectory(string path)
		{
			string[] fileList = Directory.GetFiles(path);

			foreach( string file in fileList ) 
			{
				FileUtils.DeleteFile(file);
			}

			string[] subDirList = Directory.GetDirectories(path);
			
			foreach( string subDir in subDirList )
			{
				FileUtils.DeleteDirectory(subDir);
			}

			FileUtils.ChangeAttributesToWritable( path );
			Directory.Delete( path, true );	
		}

		public static void DeleteFileOrDirectory(string path)
		{
			if ( FileUtils.IsDirectory(path) )
			{
				FileUtils.DeleteDirectory(path);
			}
			else
			{
				FileUtils.DeleteFile(path);
			}
		}

		public static void RegisterExtension(string extension, 
			string path, string fileType, string description)
		{
            StringBuilder commandline = new StringBuilder();

            commandline.Append( PathUtils.DoubleQuotString(path) );
            commandline.Append( " " );
            commandline.Append( PathUtils.DoubleQuotString("%1") );
 
			//t@C^Cvo^
			Microsoft.Win32.RegistryKey regkey =
				Microsoft.Win32.Registry.ClassesRoot.CreateSubKey(
				extension);
			regkey.SetValue("", fileType);
			regkey.Close();

			//t@C^CvƂ̐o^
			Microsoft.Win32.RegistryKey shellkey =
				Microsoft.Win32.Registry.ClassesRoot.CreateSubKey(
				fileType);
			shellkey.SetValue("", description);

			// c
			Microsoft.Win32.RegistryKey restKey = 
				shellkey.CreateSubKey("shell\\open\\command");
			restKey.SetValue( "", commandline.ToString() );
			restKey.Close();
		}

		public static void UnRegisterExtension(string extension, 
			string fileType)
		{
			Microsoft.Win32.Registry.ClassesRoot.DeleteSubKeyTree(extension);
			Microsoft.Win32.Registry.ClassesRoot.DeleteSubKeyTree(fileType);
		}
	}
}
